import pyautd3

from pyautd3_link_soem.native_methods.autd3_link_soem import ProcessPriority, SyncMode, TimerStrategy
from pyautd3_link_soem.native_methods.autd3capi_link_soem import NativeMethods as Soem

from .adapter import EtherCATAdapter
from .local import SOEM, SOEMOption
from .remote import RemoteSOEM
from .status import Status
from .thread_priority import ThreadPriority

pyautd3._ext_tracing_init.append(lambda: Soem().link_soem_tracing_init())


__all__ = [
    "SOEM",
    "EtherCATAdapter",
    "ProcessPriority",
    "RemoteSOEM",
    "SOEMOption",
    "Status",
    "SyncMode",
    "ThreadPriority",
    "TimerStrategy",
]


__version__ = "32.1.0"



from pyautd3.native_methods.utils import ConstantADT

from pyautd3_link_soem.native_methods.autd3capi_link_soem import NativeMethods as LinkSOEM
from pyautd3_link_soem.native_methods.autd3capi_link_soem import ThreadPriorityPtr


class ThreadPriority(metaclass=ConstantADT):
    Min: ThreadPriorityPtr = LinkSOEM().link_soem_thread_priority_min()
    Max: ThreadPriorityPtr = LinkSOEM().link_soem_thread_priority_max()

    @staticmethod
    def Crossplatform(value: int) -> ThreadPriorityPtr:  # noqa: N802
        if not (0 <= value <= 99):  # noqa: PLR2004
            msg = "value must be between 0 and 99"
            raise ValueError(msg)
        return LinkSOEM().link_soem_thread_priority_crossplatform(value)



import enum


class Status(enum.IntEnum):
    Error = 0
    Lost = 1
    StateChanged = 2

    @classmethod
    def from_param(cls, obj):
        return int(obj)  # pragma: no cover


class ProcessPriority(enum.IntEnum):
    Idle = 0
    BelowNormal = 1
    Normal = 2
    AboveNormal = 3
    High = 4
    Realtime = 5

    @classmethod
    def from_param(cls, obj):
        return int(obj)  # pragma: no cover


class SyncMode(enum.IntEnum):
    DC = 0
    FreeRun = 1

    @classmethod
    def from_param(cls, obj):
        return int(obj)  # pragma: no cover


class TimerStrategy(enum.IntEnum):
    SpinSleep = 0
    StdSleep = 1
    SpinWait = 2

    @classmethod
    def from_param(cls, obj):
        return int(obj)  # pragma: no cover
